// Copyright (c) 2002 Andreas Jnsson, www.AngelCode.com

// Don't forget to link with winhook.lib


/************************************************
Modified by Chris Grams to eliminate the need for
windows message hooks

instead of installing message hooks, what I have done here is
used the RedrawWindow function to update the parts of the desktop
the need updating.

this erases everything inside the rect rc and the RDW_UPDATENOW flag
tells RedrawWindow that the window should be redrawn immediately
RedrawWindow( hDesktopWnd, &rc, NULL, RDW_INVALIDATE | RDW_ERASE | RDW_UPDATENOW );

this just draws everything thats over top of the desktop background.
RDW_NOERASE tells RedrawWindow not to have the desktop erase the background
which essentially does everything that the windows hooks did, only without
the windows hooks.
RedrawWindow( hDesktopWnd, &rc, NULL, RDW_NOERASE | RDW_INVALIDATE | RDW_UPDATENOW );
*************************************************/


#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <math.h>
#include "resource.h"

// Global Variables:
HWND    hDesktopWnd = 0;
HWND    hWnd        = 0;
int     posX        = 0;
int     posY        = 0;
float   angle       = 0;

// Foward declarations of functions included in this code module:
BOOL			 WindowCreate(HINSTANCE);
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
INT_PTR CALLBACK DialogProc(HWND, UINT, WPARAM, LPARAM);

void             DrawDesktop();

// Main function
int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	// Create our window:
	if (!WindowCreate(hInstance)) 
		return FALSE;

	// Find the desktop window
	HWND hProgMan  = FindWindow("ProgMan", NULL);	
	HWND hShellDef = FindWindowEx(hProgMan, NULL, "SHELLDLL_DefView", NULL);
	hDesktopWnd    = FindWindowEx(hShellDef, NULL, "SysListView32", NULL);

	// Main message loop:
	MSG msg;
	while( GetMessage(&msg, NULL, 0, 0) ) 
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	// Do a last invalidation to remove the drawing
	RECT rc = {posX - 50, posY - 50, posX+50, posY+50};
	InvalidateRect(hDesktopWnd, &rc, TRUE);

	return msg.wParam;
}

// Window creation code
BOOL WindowCreate(HINSTANCE hInstance)
{
	// Create our dialog that we will use for main window
	hWnd = CreateDialog(hInstance, MAKEINTRESOURCE(IDD_DIALOG), 0, (DLGPROC)DialogProc);
	if (!hWnd)
	{
		char *buff;
		DWORD err = GetLastError();
		FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM, 0, err, 0, (char*)&buff, 256, 0);

		MessageBox( NULL, (LPCTSTR)buff, "Error", MB_OK | MB_ICONINFORMATION );

		LocalFree( buff );

		return FALSE;
	}

	// We don't want to show our window
	ShowWindow(hWnd, SW_SHOW);

	return TRUE;
}

// Our dialog procedure
INT_PTR CALLBACK DialogProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) 
	{
	case WM_COMMAND:
		// Close the window no matter what command it is
		DestroyWindow(hWnd);
		return TRUE;

	case WM_INITDIALOG:
		{
			// Set the small icon
			HICON iconSmall = (HICON)LoadImage(GetModuleHandle(0), MAKEINTRESOURCE(IDI_SMALL), IMAGE_ICON, 0, 0, LR_DEFAULTSIZE|LR_SHARED);
			SendMessage(hWnd, WM_SETICON, ICON_SMALL, (LPARAM)iconSmall);

			// Create a timer so that we can animate our drawing
			SetTimer(hWnd, 1, 50, 0);
		}
		return TRUE;

	case WM_CLOSE:
		// Destroy the window
		DestroyWindow(hWnd);
		return TRUE;

	case WM_DESTROY:
		// Kill our timer
		KillTimer(hWnd, 1);

		// Post the quit message so that our main loop will know that we have quit
		PostQuitMessage(0);
		return TRUE;

	case WM_TIMER:
		// Animate the drawing
		{
			angle += 0.1f;
			if( angle >= 2*3.141592f ) angle -= 2*3.141592f;

			// Get the current mouse position
			POINT pt;
			GetCursorPos(&pt);

			// Update position
			RECT rc = {posX-50, posY-50, posX+50, posY+50};

			// redraw the desktop window right away
			RedrawWindow( hDesktopWnd, &rc, NULL, RDW_INVALIDATE | RDW_ERASE | RDW_UPDATENOW );

			posX = pt.x;
			posY = pt.y;


			// are we re-drawing the window at the new cursor position just to make sure that
			// the part of the window under the animation is validated?!
			RECT rc2 = {posX-50, posY-50, posX+50, posY+50};
			RedrawWindow( hDesktopWnd, &rc, NULL, RDW_INVALIDATE | RDW_ERASE | RDW_UPDATENOW );

			DrawDesktop();
		}
		return TRUE;
   }

   return FALSE;
}

// This is where we draw our image on the desktop
void DrawDesktop()
{
	// Get the DC to the desktop window
	HDC dc = GetDC(hDesktopWnd);

	// Store DC settings
	int storedDC = SaveDC(dc);

	// Draw the rotating star
	POINT pts[5];
	pts[0].x = posX + int(50*cos(angle + 0.000)); pts[0].y = posY + int(50*sin(angle + 0.000));
	pts[1].x = posX + int(50*cos(angle + 2.513)); pts[1].y = posY + int(50*sin(angle + 2.513));
	pts[2].x = posX + int(50*cos(angle + 5.027)); pts[2].y = posY + int(50*sin(angle + 5.027));
	pts[3].x = posX + int(50*cos(angle + 1.257)); pts[3].y = posY + int(50*sin(angle + 1.257));
	pts[4].x = posX + int(50*cos(angle + 3.770)); pts[4].y = posY + int(50*sin(angle + 3.770));
	Polygon(dc, pts, 5);

	// redraw the portion of the window that was just painted over with the rotating star,
	// but specify RDW_NOERASE to keep the desktop from drawing the background as well
	RECT rc = {posX-50, posY-50, posX+50, posY+50};
	RedrawWindow( hDesktopWnd, &rc, NULL, RDW_NOERASE | RDW_INVALIDATE | RDW_UPDATENOW );

	// Restore DC settings to their original values
	RestoreDC(dc, storedDC);

	// Release the DC
	ReleaseDC(hDesktopWnd, dc);
}